@@ -92,7 +92,8 @@ def post_status(
     media_ids=None,
     sensitive=False,
     spoiler_text=None,
-    in_reply_to_id=None
+    in_reply_to_id=None,
+    language=None,
 ):
     """
     Posts a new status.
@@ -110,6 +111,7 @@ def post_status(
         'sensitive': str_bool(sensitive),
         'spoiler_text': spoiler_text,
         'in_reply_to_id': in_reply_to_id,
+        'language': language,
     }, headers=headers).json()
 
 
@@ -260,6 +262,10 @@ def get_notifications(app, user):
     return http.get(app, user, '/api/v1/notifications').json()
 
 
+def clear_notifications(app, user):
+    http.post(app, user, '/api/v1/notifications/clear')
+
+
 def get_instance(domain, scheme="https"):
     url = "{}://{}/api/v1/instance".format(scheme, domain)
     return http.anon_get(url).json()
