@@ -3,11 +3,12 @@
 from toot import api, config
 from toot.auth import login_interactive, login_browser_interactive, create_app_interactive
 from toot.exceptions import ConsoleError, NotFoundError
-from toot.output import print_out, print_instance, print_account, print_search_results, print_timeline
+from toot.output import (print_out, print_instance, print_account,
+                         print_search_results, print_timeline, print_notifications)
 from toot.utils import assert_domain_exists, multiline_input, EOF_KEY
 
 
-def timeline(app, user, args):
+def get_timeline_generator(app, user, args):
     # Make sure tag, list and public are not used simultaneously
     if len([arg for arg in [args.tag, args.list, args.public] if arg]) > 1:
         raise ConsoleError("Only one of --public, --tag, or --list can be used at one time.")
@@ -15,46 +16,78 @@ def timeline(app, user, args):
     if args.local and not (args.public or args.tag):
         raise ConsoleError("The --local option is only valid alongside --public or --tag.")
 
-    if args.public:
-        items = api.timeline_public(app, user, local=args.local)
-    elif args.tag:
-        items = api.timeline_tag(app, user, args.tag, local=args.local)
-    elif args.list:
-        items = api.timeline_list(app, user, args.list)
-    else:
-        items = api.timeline_home(app, user)
-
-    if args.reverse:
-        items = reversed(items)
-
-    print_timeline(items)
-
-
-def curses(app, user, args):
-    from toot.ui.app import TimelineApp
-
-    if not args.public and (not app or not user):
-        raise ConsoleError("You must be logged in to view the home timeline.")
+    if args.instance and not (args.public or args.tag):
+        raise ConsoleError("The --instance option is only valid alongside --public or --tag.")
 
     if args.public:
         instance = args.instance or app.instance
-        generator = api.public_timeline_generator(instance)
+        return api.public_timeline_generator(instance, local=args.local, limit=args.count)
+    elif args.tag:
+        instance = args.instance or app.instance
+        return api.tag_timeline_generator(instance, args.tag, local=args.local, limit=args.count)
+    elif args.list:
+        return api.timeline_list_generator(app, user, args.list, limit=args.count)
     else:
-        generator = api.home_timeline_generator(app, user)
+        return api.home_timeline_generator(app, user, limit=args.count)
 
-    TimelineApp(generator).run()
+
+def timeline(app, user, args):
+    generator = get_timeline_generator(app, user, args)
+
+    while(True):
+        try:
+            items = next(generator)
+        except StopIteration:
+            print_out("That's all folks.")
+            return
+
+        if args.reverse:
+            items = reversed(items)
+
+        print_timeline(items)
+
+        if args.once:
+            break
+
+        char = input("\nContinue? [Y/n] ")
+        if char.lower() == "n":
+            break
+
+
+def thread(app, user, args):
+    toot = api.single_status(app, user, args.status_id)
+    context = api.context(app, user, args.status_id)
+    thread = []
+    for item in context['ancestors']:
+        thread.append(item)
+
+    thread.append(toot)
+
+    for item in context['descendants']:
+        thread.append(item)
+
+    print_timeline(thread)
+
+
+def curses(app, user, args):
+    generator = get_timeline_generator(app, user, args)
+    from toot.ui.app import TimelineApp
+    TimelineApp(app, user, generator).run()
 
 
 def post(app, user, args):
+    if args.media and len(args.media) > 4:
+        raise ConsoleError("Cannot attach more than 4 files.")
+
     if args.media:
-        media = _do_upload(app, user, args.media)
-        media_ids = [media['id']]
+        media = [_do_upload(app, user, file) for file in args.media]
+        media_ids = [m["id"] for m in media]
     else:
         media = None
         media_ids = None
 
     if media and not args.text:
-        args.text = media['text_url']
+        args.text = "\n".join(m['text_url'] for m in media)
 
     if not args.text:
         print_out("Write or paste your toot. Press <yellow>{}</yellow> to post it.".format(EOF_KEY))
@@ -70,6 +103,7 @@ def post(app, user, args):
         sensitive=args.sensitive,
         spoiler_text=args.spoiler_text,
         in_reply_to_id=args.reply_to,
+        language=args.language,
     )
 
     print_out("Toot posted: <green>{}</green>".format(response.get('url')))
@@ -110,6 +144,11 @@ def unpin(app, user, args):
     print_out("<green>✓ Status unpinned</green>")
 
 
+def reblogged_by(app, user, args):
+    for account in api.reblogged_by(app, user, args.status_id):
+        print_out("{}\n @{}".format(account['display_name'], account['acct']))
+
+
 def auth(app, user, args):
     config_data = config.load_config()
 
@@ -259,3 +298,17 @@ def instance(app, user, args):
             "Instance not found at {}.\n"
             "The given domain probably does not host a Mastodon instance.".format(name)
         )
+
+
+def notifications(app, user, args):
+    if args.clear:
+        api.clear_notifications(app, user)
+        print_out("<green>Cleared notifications</green>")
+        return
+
+    notifications = api.get_notifications(app, user)
+    if not notifications:
+        print_out("<yellow>No notification</yellow>")
+        return
+
+    print_notifications(notifications)
