@@ -4,7 +4,7 @@ import os
 import sys
 import logging
 
-from argparse import ArgumentParser, FileType
+from argparse import ArgumentParser, FileType, ArgumentTypeError
 from collections import namedtuple
 from toot import config, commands, CLIENT_NAME, CLIENT_WEBSITE, __version__
 from toot.exceptions import ApiError, ConsoleError
@@ -13,14 +13,32 @@ from toot.output import print_out, print_err
 VISIBILITY_CHOICES = ['public', 'unlisted', 'private', 'direct']
 
 
+def language(value):
+    """Validates the language parameter"""
+    if len(value) != 3:
+        raise ArgumentTypeError(
+            "Invalid language specified: '{}'. Expected a 3 letter "
+            "abbreviation according to ISO 639-2 standard.".format(value)
+        )
+
+    return value
+
+
 def visibility(value):
-    """Validates the visibilty parameter"""
+    """Validates the visibility parameter"""
     if value not in VISIBILITY_CHOICES:
         raise ValueError("Invalid visibility value")
 
     return value
 
 
+def timeline_count(value):
+    n = int(value)
+    if not 0 < n <= 20:
+        raise ArgumentTypeError("Number of toots should be between 1 and 20.")
+    return n
+
+
 Command = namedtuple("Command", ["name", "description", "require_auth", "arguments"])
 
 
@@ -77,6 +95,58 @@ status_id_arg = (["status_id"], {
     "type": int,
 })
 
+# Arguments for selecting a timeline (see `toot.commands.get_timeline_generator`)
+common_timeline_args = [
+    (["-p", "--public"], {
+        "action": "store_true",
+        "default": False,
+        "help": "show public timeline (does not require auth)",
+    }),
+    (["-t", "--tag"], {
+        "type": str,
+        "help": "show hashtag timeline (does not require auth)",
+    }),
+    (["-l", "--local"], {
+        "action": "store_true",
+        "default": False,
+        "help": "show only statuses from local instance (public and tag timelines only)",
+    }),
+    (["-i", "--instance"], {
+        "type": str,
+        "help": "mastodon instance from which to read (public and tag timelines only)",
+    }),
+    (["--list"], {
+        "type": int,
+        "help": "show timeline for given list.",
+    }),
+]
+
+timeline_args = common_timeline_args + [
+    (["-c", "--count"], {
+        "type": timeline_count,
+        "help": "number of toots to show per page (1-20, default 10).",
+        "default": 10,
+    }),
+    (["-r", "--reverse"], {
+        "action": "store_true",
+        "default": False,
+        "help": "Reverse the order of the shown timeline (to new posts at the bottom)",
+    }),
+    (["-1", "--once"], {
+        "action": "store_true",
+        "default": False,
+        "help": "Only show the first <count> toots, do not prompt to continue.",
+    }),
+]
+
+curses_args = common_timeline_args + [
+    (["-c", "--count"], {
+        "type": timeline_count,
+        "help": "number of toots to show per page (1-20, default 20).",
+        "default": 20,
+    }),
+]
+
 AUTH_COMMANDS = [
     Command(
         name="login",
@@ -110,6 +180,7 @@ AUTH_COMMANDS = [
     ),
 ]
 
+
 READ_COMMANDS = [
     Command(
         name="whoami",
@@ -127,6 +198,18 @@ READ_COMMANDS = [
         ],
         require_auth=True,
     ),
+    Command(
+        name="notifications",
+        description="Notifications for logged in user",
+        arguments=[
+            (["--clear"], {
+                "help": "delete all notifications from the server",
+                "action": 'store_true',
+                "default": False,
+            }),
+        ],
+        require_auth=True,
+    ),
     Command(
         name="instance",
         description="Display instance details",
@@ -155,49 +238,25 @@ READ_COMMANDS = [
         require_auth=True,
     ),
     Command(
-        name="timeline",
-        description="Show recent items in a timeline (home by default)",
+        name="thread",
+        description="Show toot thread items",
         arguments=[
-            (["-p", "--public"], {
-                "action": "store_true",
-                "default": False,
-                "help": "Show public timeline.",
-            }),
-            (["-t", "--tag"], {
-                "type": str,
-                "help": "Show timeline for given hashtag.",
-            }),
-            (["-i", "--list"], {
-                "type": int,
-                "help": "Show timeline for given list ID.",
-            }),
-            (["-l", "--local"], {
-                "action": "store_true",
-                "default": False,
-                "help": "Show only statuses from local instance (public and tag timelines only).",
-            }),
-            (["-r", "--reverse"], {
-                "action": "store_true",
-                "default": False,
-                "help": "Reverse the order of the shown timeline (to new posts at the bottom)",
+            (["status_id"], {
+                "help": "Show thread for toot.",
             }),
         ],
         require_auth=True,
     ),
+    Command(
+        name="timeline",
+        description="Show recent items in a timeline (home by default)",
+        arguments=timeline_args,
+        require_auth=True,
+    ),
     Command(
         name="curses",
         description="An experimental timeline app (doesn't work on Windows)",
-        arguments=[
-            (["-p", "--public"], {
-                "action": 'store_true',
-                "default": False,
-                "help": "Resolve non-local accounts",
-            }),
-            (["-i", "--instance"], {
-                "type": str,
-                "help": 'instance from which to read (for public timeline only)',
-            })
-        ],
+        arguments=curses_args,
         require_auth=False,
     ),
 ]
@@ -212,8 +271,10 @@ POST_COMMANDS = [
                 "nargs": "?",
             }),
             (["-m", "--media"], {
-                "type": FileType('rb'),
-                "help": "path to the media file to attach"
+                "action": "append",
+                "type": FileType("rb"),
+                "help": "path to the media file to attach (specify multiple "
+                        "times to attach up to 4 files)"
             }),
             (["-v", "--visibility"], {
                 "type": visibility,
@@ -227,11 +288,15 @@ POST_COMMANDS = [
             }),
             (["-p", "--spoiler-text"], {
                 "type": str,
-                "help": 'text to be shown as a warning before the actual content',
+                "help": "text to be shown as a warning before the actual content",
             }),
             (["-r", "--reply-to"], {
                 "type": int,
-                "help": 'local ID of the status you want to reply to',
+                "help": "local ID of the status you want to reply to",
+            }),
+            (["-l", "--language"], {
+                "type": language,
+                "help": "ISO 639-2 language code of the toot, to skip automatic detection",
             }),
         ],
         require_auth=True,
@@ -280,6 +345,12 @@ STATUS_COMMANDS = [
         arguments=[status_id_arg],
         require_auth=True,
     ),
+    Command(
+        name="reblogged_by",
+        description="Show accounts that reblogged the status",
+        arguments=[status_id_arg],
+        require_auth=False,
+    ),
     Command(
         name="pin",
         description="Pin a status",
