@@ -1,18 +1,19 @@
 # -*- coding: utf-8 -*-
-
+import os
 import webbrowser
 
 from toot import __version__, api
 
 from toot.exceptions import ConsoleError
 from toot.ui.parsers import parse_status
-from toot.ui.utils import draw_horizontal_divider, draw_lines
+from toot.ui.utils import draw_horizontal_divider, draw_lines, size_as_drawn
 from toot.wcstring import fit_text
 
 # Attempt to load curses, which is not available on windows
 try:
     import curses
     import curses.panel
+    import curses.textpad
 except ImportError:
     raise ConsoleError("Curses is not available on this platform")
 
@@ -46,13 +47,18 @@ class Color:
 class HeaderWindow:
     def __init__(self, stdscr, height, width, y, x):
         self.window = stdscr.subwin(height, width, y, x)
+        self.window.bkgdset(' ', Color.WHITE_ON_BLUE)
         self.height = height
         self.width = width
 
-    def draw(self):
+    def draw(self, user):
+        username = "{}@{}".format(user.username, user.instance)
+
         self.window.erase()
-        self.window.addstr(0, 1, "toot - your Mastodon command line interface", Color.YELLOW)
-        self.window.addstr(1, 1, "https://github.com/ihabunek/toot")
+        self.window.addstr("  toot", curses.A_BOLD)
+        self.window.addstr(" | ")
+        self.window.addstr(username)
+        self.window.addstr(" | ")
         self.window.refresh()
 
 
@@ -95,6 +101,9 @@ class StatusListWindow:
         self.pad = curses.newpad(10, width)
         self.pad.box()
 
+        # Make curses interpret escape sequences for getch (why is this off by default?)
+        self.pad.keypad(True)
+
         self.scroll_pos = 0
 
     def draw_statuses(self, statuses, selected, starting=0):
@@ -245,13 +254,14 @@ class StatusDetailWindow:
 
 
 class Modal:
-    def __init__(self, stdscr):
-        height, width, y, x = self.get_size_pos(stdscr)
+    def __init__(self, stdscr, resize_callback=None):
+        self.stdscr = stdscr
+        self.resize_callback = resize_callback
 
-        self.window = curses.newwin(height, width, y, x)
-        self.draw()
+        self.setup_windows()
+        self.full_redraw()
         self.panel = curses.panel.new_panel(self.window)
-        self.panel.hide()
+        self.hide()
 
     def get_content(self):
         raise NotImplementedError()
@@ -268,8 +278,12 @@ class Modal:
 
         return height, width, y, x
 
-    def draw(self):
-        self.window.erase()
+    def setup_windows(self):
+        height, width, y, x = self.get_size_pos(self.stdscr)
+        self.window = curses.newwin(height, width, y, x)
+
+    def full_redraw(self):
+        self.setup_windows()
         self.window.box()
         draw_lines(self.window, self.get_content(), 1, 2, Color.WHITE)
 
@@ -286,9 +300,16 @@ class Modal:
     def loop(self):
         self.show()
 
-        key = None
-        while key != 'q':
-            key = self.window.getkey()
+        while True:
+            ch = self.window.getch()
+            key = chr(ch).lower() if curses.ascii.isprint(ch) else None
+
+            if key == 'q':
+                break
+            elif ch == curses.KEY_RESIZE:
+                if self.resize_callback:
+                    self.resize_callback()
+                self.full_redraw()
 
         self.hide()
 
@@ -306,6 +327,8 @@ class HelpModal(Modal):
             "  v      - view current toot in browser",
             "  b      - toggle boost status",
             "  f      - toggle favourite status",
+            "  c      - post a new status",
+            "  r      - reply to status",
             "  q      - quit application",
             "  s      - show sensitive content"
             "",
@@ -315,6 +338,172 @@ class HelpModal(Modal):
         ]
 
 
+class EntryModal(Modal):
+    def __init__(self, stdscr, title, footer=None, size=(None, None), default=None, resize_callback=None):
+        self.stdscr = stdscr
+        self.resize_callback = resize_callback
+        self.content = [] if default is None else default.split()
+        self.cursor_pos = 0
+        self.pad_y, self.pad_x = 2, 2
+
+        self.title = title
+        self.footer = footer
+        self.size = size
+        if self.footer:
+            self.pad_y += 1
+
+        self.setup_windows()
+        self.full_redraw()
+        self.panel = curses.panel.new_panel(self.window)
+        self.hide()
+
+    def get_size_pos(self, stdscr):
+        screen_height, screen_width = stdscr.getmaxyx()
+        if self.size[0]:
+            height = self.size[0] + (self.pad_y * 2) + 1
+        else:
+            height = int(screen_height / 1.33)
+        if self.size[1]:
+            width = self.size[1] + (self.pad_x * 2) + 1
+        else:
+            width = int(screen_width / 1.25)
+
+        y = (screen_height - height) // 2
+        x = (screen_width - width) // 2
+
+        return height, width, y, x
+
+    def setup_windows(self):
+        height, width, y, x = self.get_size_pos(self.stdscr)
+
+        self.window = curses.newwin(height, width, y, x)
+        self.text_window = self.window.derwin(height - (self.pad_y * 2), width - (self.pad_x * 2), self.pad_y, self.pad_x)
+        self.text_window.keypad(True)
+
+    def full_redraw(self):
+        self.window.erase()
+        self.window.box()
+
+        draw_lines(self.window, ["{}  (^D to confirm):".format(self.title)], 1, 2, Color.WHITE)
+        if self.footer:
+            window_height, window_width = self.window.getmaxyx()
+            draw_lines(self.window, [self.footer], window_height - self.pad_y + 1, 2, Color.WHITE)
+
+        self.window.refresh()
+        self.refresh_text()
+
+    def refresh_text(self):
+        text = self.get_content()
+        lines = text.split('\n')
+        draw_lines(self.text_window, lines, 0, 0, Color.WHITE)
+
+        text_window_height, text_window_width = self.text_window.getmaxyx()
+        text_on_screen = (''.join(self.content)[:self.cursor_pos] + '_').split('\n')
+        y, x = size_as_drawn(text_on_screen, text_window_width)
+        self.text_window.move(y, x)
+
+    def show(self):
+        super().show()
+        self.refresh_text()
+
+    def clear(self):
+        self.content = []
+        self.cursor_pos = 0
+
+    def on_resize(self):
+        if self.resize_callback:
+            self.resize_callback()
+        self.setup_windows()
+        self.full_redraw()
+
+    def do_command(self, ch):
+        if curses.ascii.isprint(ch) or ch == curses.ascii.LF:
+            text_window_height, text_window_width = self.text_window.getmaxyx()
+            y, x = size_as_drawn((self.get_content() + chr(ch)).split('\n'), text_window_width)
+            if y < text_window_height - 1 and x < text_window_width:
+                self.content.insert(self.cursor_pos, chr(ch))
+                self.cursor_pos += 1
+            else:
+                curses.beep()
+
+        elif ch == curses.KEY_BACKSPACE:
+            if self.cursor_pos > 0:
+                del self.content[self.cursor_pos - 1]
+                self.cursor_pos -= 1
+            else:
+                curses.beep()
+
+        elif ch == curses.KEY_DC:
+            if self.cursor_pos >= 0 and self.cursor_pos < len(self.content):
+                del self.content[self.cursor_pos]
+            else:
+                curses.beep()
+
+        elif ch == curses.KEY_LEFT:
+            if self.cursor_pos > 0:
+                self.cursor_pos -= 1
+            else:
+                curses.beep()
+
+        elif ch == curses.KEY_RIGHT:
+            if self.cursor_pos + 1 <= len(self.content):
+                self.cursor_pos += 1
+            else:
+                curses.beep()
+
+        elif ch in (curses.ascii.EOT, curses.ascii.RS):  # ^D or (for some terminals) Ctrl+Enter
+            return False, False
+
+        elif ch == curses.ascii.ESC:
+            self.clear()
+            return False, True
+
+        elif ch == curses.KEY_RESIZE:
+            self.on_resize()
+            return True, False
+
+        self.refresh_text()
+        return True, False
+
+    def get_content(self):
+        return ''.join(self.content)
+
+    def loop(self):
+        self.show()
+        while True:
+            ch = self.text_window.getch()
+            if not ch:
+                continue
+            should_continue, abort_flag = self.do_command(ch)
+            if not should_continue:
+                break
+        self.hide()
+        if abort_flag:
+            return None
+        else:
+            return self.get_content()
+
+
+class ComposeModal(EntryModal):
+    def __init__(self, stdscr, default_cw=None, **kwargs):
+        super().__init__(stdscr, title="Compose a toot", footer="^D to submit, ESC to quit, ^W to mark sensitive (cw)", **kwargs)
+        self.cw = default_cw
+        self.cwmodal = EntryModal(stdscr, title="Content warning", size=(1, 60), default=self.cw, resize_callback=self.on_resize)
+
+    def do_command(self, ch):
+        if ch == curses.ascii.ctrl(ord('w')):
+            self.cwmodal.on_resize()
+            self.cw = self.cwmodal.loop() or None
+            self.full_redraw()
+            return True, False
+        else:
+            return super().do_command(ch)
+
+    def loop(self):
+        content = super().loop()
+        return content, self.cw
+
+
 class TimelineApp:
     def __init__(self, app, user, status_generator):
         self.app = app
@@ -324,6 +513,7 @@ class TimelineApp:
         self.stdscr = None
 
     def run(self):
+        os.environ.setdefault('ESCDELAY', '25')
         curses.wrapper(self._wrapped_run)
 
     def _wrapped_run(self, stdscr):
@@ -345,50 +535,61 @@ class TimelineApp:
         if screen_width < 60:
             raise ConsoleError("Terminal screen is too narrow, toot curses requires at least 60 columns to display properly.")
 
+        header_height = 1
+        footer_height = 2
+        footer_top = screen_height - footer_height
+
         left_width = max(min(screen_width // 3, 60), 30)
-        right_width = screen_width - left_width
+        main_height = screen_height - header_height - footer_height
+        main_width = screen_width - left_width
 
-        self.header = HeaderWindow(self.stdscr, 2, screen_width, 0, 0)
-        self.footer = FooterWindow(self.stdscr, 2, screen_width, screen_height - 2, 0)
-        self.left = StatusListWindow(self.stdscr, screen_height - 4, left_width, 2, 0)
-        self.right = StatusDetailWindow(self.stdscr, screen_height - 4, right_width, 2, left_width)
+        self.header = HeaderWindow(self.stdscr, header_height, screen_width, 0, 0)
+        self.footer = FooterWindow(self.stdscr, footer_height, screen_width, footer_top, 0)
+        self.left = StatusListWindow(self.stdscr, main_height, left_width, header_height, 0)
+        self.right = StatusDetailWindow(self.stdscr, main_height, main_width, header_height, left_width)
 
-        self.help_modal = HelpModal(self.stdscr)
+        self.help_modal = HelpModal(self.stdscr, resize_callback=self.on_resize)
 
     def loop(self):
         while True:
-            key = self.left.pad.getkey()
+            ch = self.left.pad.getch()
+            key = chr(ch).lower() if curses.ascii.isprint(ch) else None
 
-            if key.lower() == 'q':
+            if key == 'q':
                 return
 
-            elif key.lower() == 'h':
+            elif key == 'h':
                 self.help_modal.loop()
                 self.full_redraw()
 
-            elif key.lower() == 'v':
+            elif key == 'v':
                 status = self.get_selected_status()
                 if status:
                     webbrowser.open(status['url'])
 
-            elif key.lower() == 'j' or key == 'B':
+            elif key == 'j' or ch == curses.KEY_DOWN:
                 self.select_next()
 
-            elif key.lower() == 'k' or key == 'A':
+            elif key == 'k' or ch == curses.KEY_UP:
                 self.select_previous()
 
-            elif key.lower() == 's':
+            elif key == 's':
                 self.show_sensitive()
 
-            elif key.lower() == 'b':
+            elif key == 'b':
                 self.toggle_reblog()
 
-            elif key.lower() == 'f':
+            elif key == 'f':
                 self.toggle_favourite()
 
-            elif key == 'KEY_RESIZE':
-                self.setup_windows()
-                self.full_redraw()
+            elif key == 'c':
+                self.compose()
+
+            elif key == 'r':
+                self.reply()
+
+            elif ch == curses.KEY_RESIZE:
+                self.on_resize()
 
     def show_sensitive(self):
         status = self.get_selected_status()
@@ -396,6 +597,55 @@ class TimelineApp:
             status['show_sensitive'] = True
             self.right.draw(status)
 
+    def compose(self):
+        """Compose and submit a new status"""
+        app, user = self.app, self.user
+        if not app or not user:
+            self.footer.draw_message("You must be logged in to post", Color.RED)
+            return
+
+        compose_modal = ComposeModal(self.stdscr, resize_callback=self.on_resize)
+        content, cw = compose_modal.loop()
+        self.full_redraw()
+        if content is None:
+            return
+        elif len(content) == 0:
+            self.footer.draw_message("Status must contain content", Color.RED)
+            return
+
+        self.footer.draw_message("Submitting status...", Color.YELLOW)
+        response = api.post_status(app, user, content, spoiler_text=cw, sensitive=cw is not None)
+        status = parse_status(response)
+        self.statuses.insert(0, status)
+        self.selected += 1
+        self.left.draw_statuses(self.statuses, self.selected)
+        self.footer.draw_message("✓ Status posted", Color.GREEN)
+
+    def reply(self):
+        """Reply to the selected status"""
+        status = self.get_selected_status()
+        app, user = self.app, self.user
+        if not app or not user:
+            self.footer.draw_message("You must be logged in to reply", Color.RED)
+            return
+
+        compose_modal = ComposeModal(self.stdscr, default_cw='\n'.join(status['spoiler_text']) or None, resize_callback=self.on_resize)
+        content, cw = compose_modal.loop()
+        self.full_redraw()
+        if content is None:
+            return
+        elif len(content) == 0:
+            self.footer.draw_message("Status must contain content", Color.RED)
+            return
+
+        self.footer.draw_message("Submitting reply...", Color.YELLOW)
+        response = api.post_status(app, user, content, spoiler_text=cw, sensitive=cw is not None, in_reply_to_id=status['id'])
+        status = parse_status(response)
+        self.statuses.insert(0, status)
+        self.selected += 1
+        self.left.draw_statuses(self.statuses, self.selected)
+        self.footer.draw_message("✓ Reply posted", Color.GREEN)
+
     def toggle_reblog(self):
         """Reblog or unreblog selected status."""
         status = self.get_selected_status()
@@ -483,14 +733,19 @@ class TimelineApp:
 
         return len(statuses)
 
+    def on_resize(self):
+        self.setup_windows()
+        self.full_redraw()
+
     def full_redraw(self):
         """Perform a full redraw of the UI."""
-        self.header.draw()
-        self.draw_footer_status()
-
         self.left.draw_statuses(self.statuses, self.selected)
         self.right.draw(self.get_selected_status())
 
+        self.header.draw(self.user)
+        self.draw_footer_status()
+
+
     def redraw_after_selection_change(self, old_index, new_index):
         old_status = self.statuses[old_index]
         new_status = self.statuses[new_index]
